import csv
from decimal import Decimal
from math import floor
from pyexpat.errors import messages
from django.http import HttpResponse, HttpResponseForbidden, HttpResponseRedirect
from django.shortcuts import redirect, render

from coils.models import Coil
from quotations.models import CustomerQuoteInfo, Inquiry, Quotation
from django.core.paginator import Paginator

from django.shortcuts import render, get_object_or_404
from django.contrib.auth.decorators import login_required 

def coils(request):
    coils  = Coil.objects.all()
    data = { "coils": coils}
    return render(request,"production/coils.html", data)

def production(request):
    return render(request,"production/index.html")

def produced_items(request):
    if request.user.is_admin_user or request.user.is_super_user:
        if request.user.is_admin_user or request.user.is_super_user:
            quotations = Quotation.objects.filter(seller=request.user, status="Produced").order_by('-created_at')  # For admin and super admin, get all quotations
        else:
            # For other users (sales), filter quotations where the seller is the current user
            quotations = Quotation.objects.filter(seller=request.user, status="Produced").order_by('-created_at')
        
        # Apply filters
        code = request.GET.get('code')
        if code:
            quotations = quotations.filter(qid__icontains=code)
        
        status = request.GET.get('status')
        if status:
            quotations = quotations.filter(status__icontains=status)
        
        customer_name = request.GET.get('customer_name')
        if customer_name:
            quotations = quotations.filter(customer_info__customer_name__icontains=customer_name)
        
        from_date = request.GET.get('from_date')
        to_date = request.GET.get('to_date')
        if from_date and to_date:
            quotations = quotations.filter(created_at__range=[from_date, to_date])
        
        # Pagination
        paginator = Paginator(quotations, 10) 
        page_number = request.GET.get('page')
        page_obj = paginator.get_page(page_number)
        
        data = {
            "page_obj": page_obj,
        }
        
        if 'export' in request.GET and request.GET['export'] == 'true':
            response = HttpResponse(content_type='text/csv')
            response['Content-Disposition'] = 'attachment; filename="quotations.csv"'
            
            writer = csv.writer(response)
            writer.writerow(['QUOTATION ID', 'Customer Name', 'Customer Mobile',  'Fundi Name', 'Fundi Number', 'Delivery Location', 'Delivery Mode', 'Payment Terms', 'Lead Source', 'Seller', 'Total Amount', 'Status', 'Created At'])

            from django.core.exceptions import ObjectDoesNotExist

            for quotation in quotations:
                try:
                    customer_quote_info = CustomerQuoteInfo.objects.get(quotation=quotation)
                    customer_name = customer_quote_info.customer.customer_name if customer_quote_info.customer else ""
                    customer_mobile = customer_quote_info.customer.customer_mobile if customer_quote_info.customer else ""
                    fundi = customer_quote_info.customer.fundi if (customer_quote_info.customer and customer_quote_info.customer.fundi) else ""
                    fundi_details = customer_quote_info.customer.fundi_details if (customer_quote_info.customer and customer_quote_info.customer.fundi_details) else ""
                except ObjectDoesNotExist:
                    # If CustomerQuoteInfo does not exist, set its info and customer's info to ""
                    customer_quote_info = CustomerQuoteInfo()
                    customer_name = ""
                    customer_mobile = ""
                    fundi = ""
                    fundi_details = ""

                writer.writerow([
                    quotation.qid, 
                    customer_name,
                    customer_mobile, 
                    fundi,
                    fundi_details,
                    customer_quote_info.delivery_location if customer_quote_info.delivery_location else "",  
                    customer_quote_info.delivery_mode if customer_quote_info.delivery_mode else "",   
                    customer_quote_info.payment_terms if customer_quote_info.payment_terms else "",   
                    customer_quote_info.lead_source if customer_quote_info.lead_source else "",  
                    customer_quote_info.seller.email if customer_quote_info.seller else "",   
                    quotation.total_amount, 
                    quotation.status, 
                    quotation.created_at
                ])

            return response
    
        return render(request,"production/produced.html", data)
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")
    

def in_production(request):
    if request.user.is_admin_user or request.user.is_super_user:
        if request.user.is_admin_user or request.user.is_super_user:
            quotations = Quotation.objects.filter(status="In Production").order_by('-created_at')  # For admin and super admin, get all quotations
        else:
            # For other users (sales), filter quotations where the seller is the current user
            quotations = Quotation.objects.filter(status="In Production").order_by('-created_at')
        
        # Apply filters
        code = request.GET.get('code')
        if code:
            quotations = quotations.filter(qid__icontains=code)
        
        status = request.GET.get('status')
        if status:
            quotations = quotations.filter(status__icontains=status)
        
        customer_name = request.GET.get('customer_name')
        if customer_name:
            quotations = quotations.filter(customer_info__customer_name__icontains=customer_name)
        
        from_date = request.GET.get('from_date')
        to_date = request.GET.get('to_date')
        if from_date and to_date:
            quotations = quotations.filter(created_at__range=[from_date, to_date])
        
        # Pagination
        paginator = Paginator(quotations, 10) 
        page_number = request.GET.get('page')
        page_obj = paginator.get_page(page_number)
        
        data = {
            "page_obj": page_obj,
        }
        
        if 'export' in request.GET and request.GET['export'] == 'true':
            response = HttpResponse(content_type='text/csv')
            response['Content-Disposition'] = 'attachment; filename="quotations.csv"'
            
            writer = csv.writer(response)
            writer.writerow(['QUOTATION ID', 'Customer Name', 'Customer Mobile',  'Fundi Name', 'Fundi Number', 'Delivery Location', 'Delivery Mode', 'Payment Terms', 'Lead Source', 'Seller', 'Total Amount', 'Status', 'Created At'])

            from django.core.exceptions import ObjectDoesNotExist

            for quotation in quotations:
                try:
                    customer_quote_info = CustomerQuoteInfo.objects.get(quotation=quotation)
                    customer_name = customer_quote_info.customer.customer_name if customer_quote_info.customer else ""
                    customer_mobile = customer_quote_info.customer.customer_mobile if customer_quote_info.customer else ""
                    fundi = customer_quote_info.customer.fundi if (customer_quote_info.customer and customer_quote_info.customer.fundi) else ""
                    fundi_details = customer_quote_info.customer.fundi_details if (customer_quote_info.customer and customer_quote_info.customer.fundi_details) else ""
                except ObjectDoesNotExist:
                    # If CustomerQuoteInfo does not exist, set its info and customer's info to ""
                    customer_quote_info = CustomerQuoteInfo()
                    customer_name = ""
                    customer_mobile = ""
                    fundi = ""
                    fundi_details = ""

                writer.writerow([
                    quotation.qid, 
                    customer_name,
                    customer_mobile, 
                    fundi,
                    fundi_details,
                    customer_quote_info.delivery_location if customer_quote_info.delivery_location else "",  
                    customer_quote_info.delivery_mode if customer_quote_info.delivery_mode else "",   
                    customer_quote_info.payment_terms if customer_quote_info.payment_terms else "",   
                    customer_quote_info.lead_source if customer_quote_info.lead_source else "",  
                    customer_quote_info.seller.email if customer_quote_info.seller else "",   
                    quotation.total_amount, 
                    quotation.status, 
                    quotation.created_at
                ])

            return response
    
        return render(request,"production/in_production.html", data)
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")
    


@login_required
def quotation_detail(request, id):
    quotation = get_object_or_404(Quotation, id=id)
    customer_quote_info = CustomerQuoteInfo.objects.filter(quotation=quotation).first()
    inquiries = Inquiry.objects.filter(quotation=quotation)
    context = {
        'quotation': quotation,
        'customer_quote_info': customer_quote_info,
        'inquiries': inquiries,
    }
    return render(request, 'production/quotation_detail.html', context)

# @login_required
# def quotation_production(request, id):
#     quotation = get_object_or_404(Quotation, id=id)
#     inquiries = Inquiry.objects.filter(quotation=quotation)
#     coils = Coil.objects.filter(is_allocated=False, is_blocked=False)
#     return render(request, 'production/quotation_production.html', {'quotation': quotation, 'inquiries': inquiries, 'coils': coils})

@login_required
def allocate_coil(request, inquiry_id):
    inquiry = get_object_or_404(Inquiry, id=inquiry_id)
    unallocated_coils = Coil.objects.filter(is_allocated=False, is_blocked=False)
    
    if request.method == 'POST':
        coil = unallocated_coils.first()  # Just take the first unallocated coil for simplicity
        if coil:
            coil.is_allocated = True
            coil.save()
            # Logic to associate coil with inquiry can be added here
            # You may want to create a new model to track this relationship if needed

    return redirect('production:quotation_production', id=inquiry.quotation.id)

@login_required
def update_coil_usage(request):
    if request.method == 'POST':
        for key, value in request.POST.items():
            if key.startswith('meters_used_'):
                coil_id = key.split('_')[2]
                coil = Coil.objects.get(id=coil_id)
                meters_used = Decimal(value)
                rejects = Decimal(request.POST.get(f'rejects_{coil_id}', '0'))
                waste_length = Decimal(request.POST.get(f'waste_length_{coil_id}', '0'))

                total_used = meters_used + rejects + waste_length
                coil.meters -= total_used
                coil.save()

                # Update the inquiry produced quantity and status
                inquiry = Inquiry.objects.get(id=coil.inquiry.id)
                inquiry.produced_quantity += meters_used
                if inquiry.produced_quantity >= inquiry.quantity:
                    inquiry.status = 'Produced'
                inquiry.save()

    return redirect('production:quotation_production', id=request.POST.get('quotation_id'))


def update_quotation_status(quotation):
    """
    Update the status of the Quotation based on the status of its related product inquiries.
    If all inquiries are processed, set the status to 'Produced'.
    """
    # Fetch product inquiries related to the current Quotation
    product_inquiries = Inquiry.objects.filter(quotation=quotation)

    # Check if all inquiries are processed
    if all(inquiry.production_status == 'Produced' for inquiry in product_inquiries):
        quotation.status = 'Produced'
        quotation.save()
        
        
 
from django.core.exceptions import ValidationError
from products.models import Product
def quotation_production(request, quotation_id): 
    print(quotation_id)
    # Fetch the Quotation object
    quotation = get_object_or_404(Quotation, pk=quotation_id) 
    # Fetch product inquiries related to the current Quotation and with the specified product type
    product_inquiries = Inquiry.objects.filter(quotation=quotation, product__productType="Production Item")
    
    # Print the IDs of the retrieved product inquiries
   
     

    if request.method == 'POST':
        inquiry_id = request.POST.get('inquiry_id')
        coil_id = request.POST.get('coil_id')
        used_meters = Decimal(request.POST.get('meters_used'))
        offcuts = Decimal(request.POST.get('offcuts'))
        rejects = Decimal(request.POST.get('rejects'))
        overproduction = Decimal(request.POST.get('overproduction'))
        
         
        if used_meters < 0:
            messages.error(request, 'Input must be a non-negative number.')
            # Redirect to the same quotation_production page after processing
            return HttpResponseRedirect(reverse('production:quotation_production', args=[quotation_id]))
            # print("Input must be a non-negative number.")
            # raise ValidationError("Input must be a non-negative number.")

        # Fetch the Inquiry object
        product_inquiry = get_object_or_404(Inquiry, id=inquiry_id, quotation=quotation)

        # Fetch the Coil object
        coil = get_object_or_404(Coil, id=coil_id)
        print(coil.original_meters)
        print(coil.meters_used)
        print(coil.main_meters)
        
        
        #coil.meters_used  = Decimal(coil.meters_used) + used_meters + offcuts + rejects 
        # Subtract used meters from the assigned coil 
        coil.save()

        # Subtract offcuts and rejects from the coil meters
        #coil.meters = coil.meters - (offcuts + rejects)
 
        # Check if coil meters are less than the required meters
        if coil.meters < used_meters:
            messages.error(request, 'Coil meters are less than the required meters!')
         
        
        # Check if meters used is less than 5 meters and update is_coil_requested
        if coil.meters < 5:
            product_inquiry.is_coil_requested = False
            product_inquiry.save() 

        # Convert product_inquiry.total_meters to Decimal before subtraction
        product_inquiry.total_meters = Decimal(str(product_inquiry.total_meters)) - used_meters
        product_inquiry.save()

        # Save meters used in CoilHistory
        # coil_history = ProductionHistory(inquiry=product_inquiry, 
        #                                  operator = Account.objects.get(id=request.user.id),
        #                                  quotation = Quotation.objects.get(id=quotation_id),
        #                                  coil=coil, 
        #                                  used_length=used_meters,
        #                                  offcuts=offcuts, 
        #                                  rejects=rejects, 
        #                                  overproduction=overproduction)
        # coil_history.save()
        
        # Calculate items made based on meters used and inquiry meters
        items_made = floor(used_meters / product_inquiry.meters)

         
        remaining_quantity = product_inquiry.quantity - product_inquiry.items_made

        if items_made > remaining_quantity:
            # If calculated items_made exceed the remaining quantity, set items_made to the remaining quantity
            items_made = remaining_quantity
            print("Exceeding")
            messages.warning(request, f'Items made adjusted to {items_made} to prevent exceeding the required quantity.')

        # Update items made
        product_inquiry.items_made += items_made
        product_inquiry.save()

        # Adjust the coil usage
        used_coil_meters = items_made * product_inquiry.meters
        coil.meters -= used_coil_meters
        print(coil.meters)

        # Check if meters used is less than 5 meters and update is_coil_requested
        if coil.meters < 5:
            product_inquiry.is_coil_requested = False
            product_inquiry.save()

        coil.save()

        # Check if the production is completed
        if product_inquiry.items_made == product_inquiry.quantity:
            product_inquiry.production_status = 'Produced'
            product_inquiry.save()

            # Update Quotation status
            update_quotation_status(quotation)

        elif product_inquiry.items_made > product_inquiry.quantity:
            # Adjust items_made to the remaining quantity to prevent exceeding the required quantity
            items_made = product_inquiry.quantity - product_inquiry.items_made
            print("Exceeded")
            messages.warning(request, f'Items made adjusted to {items_made} to prevent exceeding the required quantity.')

            # Update items made
            product_inquiry.items_made += items_made
            product_inquiry.save()

            # Adjust the coil usage accordingly
            used_coil_meters = items_made * product_inquiry.meters
            coil.meters -= used_coil_meters
            
            

            # Check if meters used is less than 5 meters and update is_coil_requested
            if coil.meters < 5:
                product_inquiry.is_coil_requested = False
                product_inquiry.save()

            coil.save()

            # Update Quotation status
            update_quotation_status(quotation)

            messages.error(request, 'Produced items exceed the required quantity!')
        
        
        coil = Coil.objects.get(id=coil_id)
        
        coil.meters = coil.original_meters - (used_meters + offcuts + rejects)
        coil.meters_used = coil.main_meters - coil.meters
        print(coil.meters_used)
        coil.save()
        print("Coil Meters Left : ", coil.meters) 

        # Redirect to the same quotation_production page after processing
        return HttpResponseRedirect(reverse('production:quotation_production', args=[quotation_id]))

    # Render the template with the Quotation, payments, and product inquiries
    return render(request, 'production/quotation_production.html',
                  {'quotation': quotation, 
                   'inquiries': product_inquiries})
 
from django.urls import reverse
def request_coil(request, inquiry_id, quotation_id): 
    print(quotation_id)
    print(inquiry_id)
    try:
        # Fetch the Inquiry object
        Inquiry.objects.filter(id=inquiry_id, quotation= quotation_id).update(is_coil_requested = True)
  
        # Fetch the Quotation object
        quotation = get_object_or_404(Quotation, pk=quotation_id) 

        # Fetch product inquiries related to the current Quotation
        product_inquiries = Inquiry.objects.filter(quotation=quotation)

        # Render the template with the Quotation, payments, and product inquiries
        return redirect('production:quotation_production', quotation_id=quotation_id)

    except Inquiry.DoesNotExist:
        # Handle the case where the Inquiry does not exist
        # return None
        return redirect('production:quotation_production', quotation_id=quotation_id)

from django.shortcuts import get_object_or_404, render, redirect
from coils.models import Coil 
from colors.models import Color 
from django.db.models import Q

def accept_coil_request(request, inquiry_id, quotation_id): 
    print("testing")
    try:
        # Fetch the Inquiry object
        product_inquiry = get_object_or_404(Inquiry, id=inquiry_id, quotation=quotation_id) 
        # Fetch the first available coil that is not allocated and has meters not equal to 0.0
        print(product_inquiry.color)
        coil = Coil.objects.filter(
            meters__gt=0.0, 
            status="Available", 
            color=Color.objects.filter(name = product_inquiry.color).first(),
            gauge = product_inquiry.gauge
            ).first() 
        print(coil)

        if coil:
            # Update the Inquiry with the allocated coil
            product_inquiry.is_coil_requested = True
            product_inquiry.coil.add(coil)
            product_inquiry.save()

            # Mark the allocated coil as allocated
            coil.is_allocated = True
            coil.save()

            # Fetch the Quotation object and product inquiries
            quotation = get_object_or_404(Quotation, pk=quotation_id)
            product_inquiries = Inquiry.objects.filter(quotation=quotation)

            # Render the template with the Quotation, payments, and product inquiries
            redirect('production:quotation_production', quotation_id=quotation_id)

    except Inquiry.DoesNotExist:
        return redirect('production:quotation_production', quotation_id=quotation_id)


    # Fetch the Quotation object and product inquiries
    quotation = get_object_or_404(Quotation, pk=quotation_id)
    print("Testing")
    product_inquiries = Inquiry.objects.filter(quotation=quotation)
    

    # Render the template with the Quotation, payments, and product inquiries
    return redirect('production:quotation_production', quotation_id=quotation_id)


def allow_production_view(request, quotation_id): 
    
    quotations = Quotation.objects.filter(Q(status="PAID") | Q(payment_terms="Partial Payment"))
    
    # Retrieve the quotation instance
    quotation = get_object_or_404(Quotation, id=quotation_id)
    
    data = {
        "production_quotations" : quotations, 
    }
 
    if not quotation.production_allowed and quotation in quotations:
        # Update the 'production_allowed' field
        quotation.production_allowed = True
        quotation.save()

        # Update the production status for related ProductInquiry instances
        Inquiry.objects.filter(quotation=quotation).update(production_status='In Production')

        # Redirect or render as needed
        return render(request,"production/production.html",data)
    

    return render(request,"production/production.html",data)

# views.py

# from django.shortcuts import render
# from .utils import request_coil, manage_waste, shift_remaining_coils

# def process_inquiry(request, inquiry_id):
#     # Fetch inquiry details
#     inquiry = Inquiry.objects.get(id=inquiry_id)

#     # Example: Request coil for the inquiry
#     requested_length = 4.0  # Replace with actual requested length
#     gauge = 'G28'  # Replace with actual gauge
#     request_coil(inquiry, requested_length, gauge)

#     # Example: Manage waste for the inquiry
#     remaining_length = 1.5  # Replace with actual remaining length
#     coil = Coil.objects.get(id=1)  # Replace with actual coil
#     manage_waste(coil, remaining_length)

#     # Example: Shift remaining coils to the next inquiry
#     from_inquiry = inquiry  # Replace with actual inquiry
#     to_inquiry = Inquiry.objects.get(id=2)  # Replace with actual inquiry
#     shift_remaining_coils(from_inquiry, to_inquiry)

#     return render(request, 'process_inquiry.html', {'inquiry': inquiry})
